from typing import *
import royalnet.serf.discord as rsd
import royalnet.commands as rc


class VoicestatusCommand(rc.Command):
    name: str = "voicestatus"

    description: str = "Visualizza lo stato interno dei voice player del bot."

    syntax: str = ""

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        if self.interface.name != "discord":
            raise rc.UnsupportedError("Questo comando funziona solo su Discord.")
        serf: rsd.DiscordSerf = self.interface.serf

        message = []
        for index, voice_player in enumerate(serf.voice_players):
            message.append(f"🎵 [b]Voice Player #{index}[/b]")

            voice_client = voice_player.voice_client
            if voice_client.is_connected():
                message.append(f"🔵 Connected")
            else:
                message.append(f"🔴 Disconnected")
            if voice_client.is_playing():
                message.append(f"🔵 Playing")
            else:
                message.append(f"⚪ Not playing")
            if voice_client.is_paused():
                message.append(f"⚪ Paused")
            else:
                message.append(f"🔵 Not paused")

            playable = voice_player.playing
            message.append(f"🔉 {playable.__class__.__name__}")

            message.append("")

        if len(serf.voice_players) == 0:
            message.append("[i]Nessun voice player.[/i]")

        await data.reply("\n".join(message))



from typing import *
import datetime

from sqlalchemy import *
from sqlalchemy.orm import *
from sqlalchemy.ext.declarative import declared_attr

from .fiorygi import Fiorygi

if TYPE_CHECKING:
    from royalnet.commands import CommandData


class FiorygiTransaction:
    __tablename__ = "fiorygitransactions"

    @declared_attr
    def id(self):
        return Column(Integer, primary_key=True)

    @declared_attr
    def change(self):
        return Column(Integer, nullable=False)

    @declared_attr
    def user_id(self):
        return Column(Integer, ForeignKey("fiorygi.user_id"), nullable=False)

    @declared_attr
    def wallet(self):
        return relationship("Fiorygi", backref=backref("transactions"))

    @property
    def user(self):
        return self.wallet.user

    @declared_attr
    def reason(self):
        return Column(String, nullable=False, default="")

    @declared_attr
    def timestamp(self):
        return Column(DateTime)

    def __repr__(self):
        return f"<{self.__class__.__name__}: {self.change:+} to {self.user.username} for {self.reason}>"

    @classmethod
    async def spawn_fiorygi(cls, data: "CommandData", user, qty: int, reason: str):
        if user.fiorygi is None:
            data.session.add(data._interface.alchemy.get(Fiorygi)(
                user_id=user.uid,
                fiorygi=0
            ))
            await data.session_commit()

        transaction = data._interface.alchemy.get(FiorygiTransaction)(
            user_id=user.uid,
            change=qty,
            reason=reason,
            timestamp=datetime.datetime.now()
        )
        data.session.add(transaction)

        user.fiorygi.fiorygi += qty
        await data.session_commit()

        if len(user.telegram) > 0:
            user_str = user.telegram[0].mention()
        else:
            user_str = user.username

        if qty > 0:
            msg = f"💰 [b]{user_str}[/b] ha ottenuto [b]{qty}[/b] fioryg{'i' if qty != 1 else ''} per [i]{reason}[/i]!"
        elif qty == 0:
            msg = f"❓ [b]{user_str}[/b] ha ottenuto [b]{qty}[/b] fioryg{'i' if qty != 1 else ''}...? " \
                  f"Per [i]{reason}[/i]...? Cosa?"
        else:
            msg = f"💸 [b]{user_str}[/b] ha perso [b]{-qty}[/b] fioryg{'i' if qty != -1 else ''} per [i]{reason}[/i]."

        await data._interface.call_herald_event("telegram", "telegram_message",
                                                chat_id=data._interface.config["Telegram"]["main_group_id"],
                                                text=msg)



from typing import *
import riotwatcher
import logging
import asyncio
import sentry_sdk
import royalnet.commands as rc
import royalnet.utils as ru
import royalnet.serf.telegram as rst

from ..tables import LeagueOfLegends, FiorygiTransaction
from ..types import LeagueLeague

log = logging.getLogger(__name__)


class LeagueoflegendsCommand(rc.Command):
    name: str = "leagueoflegends"

    aliases = ["lol", "league"]

    description: str = "Connetti un account di League of Legends a un account Royalnet, e visualizzane le statistiche."

    syntax = "[nomeevocatore]"

    def __init__(self, interface: rc.CommandInterface):
        super().__init__(interface)
        self._riotwatcher = riotwatcher.RiotWatcher(api_key=self.config["Lol"]["token"])
        if self.interface.name == "telegram" and self.config["Lol"]["updater"]:
            self.loop.create_task(self._updater(7200))

    async def _send(self, message):
        client = self.serf.client
        await self.serf.api_call(client.send_message,
                                 chat_id=self.config["Telegram"]["main_group_id"],
                                 text=rst.escape(message),
                                 parse_mode="HTML",
                                 disable_webpage_preview=True)

    async def _notify(self,
                      obj: LeagueOfLegends,
                      attribute_name: str,
                      old_value: Any,
                      new_value: Any):
        if isinstance(old_value, LeagueLeague):
            # This is a rank change!
            # Don't send messages for every rank change, send messages just if the TIER or RANK changes!
            if old_value.tier == new_value.tier and old_value.rank == new_value.rank:
                return
            # Find the queue
            queue_names = {
                "rank_soloq": "Solo/Duo",
                "rank_flexq": "Flex",
                "rank_twtrq": "3v3",
                "rank_tftq": "TFT"
            }
            # Prepare the message
            if new_value > old_value:
                message = f"📈 [b]{obj.user}[/b] è salito a {new_value} su League of Legends " \
                          f"({queue_names[attribute_name]})! Congratulazioni!"
            else:
                message = f"📉 [b]{obj.user}[/b] è sceso a {new_value} su League of Legends " \
                          f"({queue_names[attribute_name]})."
            # Send the message
            await self._send(message)
        # Level up!
        elif attribute_name == "summoner_level":
            if new_value == 30 or (new_value >= 50 and (new_value % 25 == 0)):
                await self._send(f"🆙 [b]{obj.user}[/b] è salito al livello [b]{new_value}[/b] su League of Legends!")

    @staticmethod
    async def _change(obj: LeagueOfLegends,
                      attribute_name: str,
                      new_value: Any,
                      callback: Callable[
                          [LeagueOfLegends, str, Any, Any], Awaitable[None]]):
        old_value = obj.__getattribute__(attribute_name)
        if old_value != new_value:
            await callback(obj, attribute_name, old_value, new_value)
        obj.__setattr__(attribute_name, new_value)

    async def _update(self, lol: LeagueOfLegends):
        log.info(f"Updating: {lol}")
        log.debug(f"Getting summoner data: {lol}")
        summoner = await ru.asyncify(self._riotwatcher.summoner.by_id, region=self.config["Lol"]["region"],
                                     encrypted_summoner_id=lol.summoner_id)
        await self._change(lol, "profile_icon_id", summoner["profileIconId"], self._notify)
        await self._change(lol, "summoner_name", summoner["name"], self._notify)
        await self._change(lol, "puuid", summoner["puuid"], self._notify)
        await self._change(lol, "summoner_level", summoner["summonerLevel"], self._notify)
        await self._change(lol, "summoner_id", summoner["id"], self._notify)
        await self._change(lol, "account_id", summoner["accountId"], self._notify)
        log.debug(f"Getting leagues data: {lol}")
        leagues = await ru.asyncify(self._riotwatcher.league.by_summoner, region=self.config["Lol"]["region"],
                                    encrypted_summoner_id=lol.summoner_id)
        soloq = LeagueLeague()
        flexq = LeagueLeague()
        twtrq = LeagueLeague()
        tftq = LeagueLeague()
        for league in leagues:
            if league["queueType"] == "RANKED_SOLO_5x5":
                soloq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_SR":
                flexq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_FLEX_TT":
                twtrq = LeagueLeague.from_dict(league)
            if league["queueType"] == "RANKED_TFT":
                tftq = LeagueLeague.from_dict(league)
        await self._change(lol, "rank_soloq", soloq, self._notify)
        await self._change(lol, "rank_flexq", flexq, self._notify)
        await self._change(lol, "rank_twtrq", twtrq, self._notify)
        await self._change(lol, "rank_tftq", tftq, self._notify)
        log.debug(f"Getting mastery data: {lol}")
        mastery = await ru.asyncify(self._riotwatcher.champion_mastery.scores_by_summoner,
                                    region=self.config["Lol"]["region"],
                                    encrypted_summoner_id=lol.summoner_id)
        await self._change(lol, "mastery_score", mastery, self._notify)

    async def _updater(self, period: int):
        log.info(f"Started updater with {period}s period")
        while True:
            log.info(f"Updating...")
            session = self.alchemy.Session()
            log.info("")
            lols = session.query(self.alchemy.get(LeagueOfLegends)).all()
            for lol in lols:
                try:
                    await self._update(lol)
                except Exception as e:
                    sentry_sdk.capture_exception(e)
                    log.error(f"Error while updating {lol.user.username}: {e}")
                await asyncio.sleep(1)
            await ru.asyncify(session.commit)
            session.close()
            log.info(f"Sleeping for {period}s")
            await asyncio.sleep(period)

    @staticmethod
    def _display(lol: LeagueOfLegends) -> str:
        string = f"ℹ️ [b]{lol.summoner_name}[/b]\n" \
                 f"Lv. {lol.summoner_level}\n" \
                 f"Mastery score: {lol.mastery_score}\n" \
                 f"\n"
        if lol.rank_soloq:
            string += f"Solo: {lol.rank_soloq}\n"
        if lol.rank_flexq:
            string += f"Flex: {lol.rank_flexq}\n"
        if lol.rank_twtrq:
            string += f"3v3: {lol.rank_twtrq}\n"
        if lol.rank_tftq:
            string += f"TFT: {lol.rank_tftq}\n"
        return string

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        author = await data.get_author(error_if_none=True)

        name = args.joined()

        if name:
            # Connect a new League of Legends account to Royalnet
            log.debug(f"Searching for: {name}")
            summoner = self._riotwatcher.summoner.by_name(region=self.config["Lol"]["region"], summoner_name=name)
            # Ensure the account isn't already connected to something else
            leagueoflegends = await ru.asyncify(
                data.session.query(self.alchemy.get(LeagueOfLegends)).filter_by(summoner_id=summoner["id"]).one_or_none)
            if leagueoflegends:
                raise rc.CommandError(f"L'account {leagueoflegends} è già registrato su Royalnet.")
            # Get rank information
            log.debug(f"Getting leagues data: {name}")
            leagues = self._riotwatcher.league.by_summoner(region=self.config["Lol"]["region"],
                                                           encrypted_summoner_id=summoner["id"])
            soloq = LeagueLeague()
            flexq = LeagueLeague()
            twtrq = LeagueLeague()
            tftq = LeagueLeague()
            for league in leagues:
                if league["queueType"] == "RANKED_SOLO_5x5":
                    soloq = LeagueLeague.from_dict(league)
                if league["queueType"] == "RANKED_FLEX_SR":
                    flexq = LeagueLeague.from_dict(league)
                if league["queueType"] == "RANKED_FLEX_TT":
                    twtrq = LeagueLeague.from_dict(league)
                if league["queueType"] == "RANKED_TFT":
                    tftq = LeagueLeague.from_dict(league)
            # Get mastery score
            log.debug(f"Getting mastery data: {name}")
            mastery = self._riotwatcher.champion_mastery.scores_by_summoner(region=self.config["Lol"]["region"],
                                                                            encrypted_summoner_id=summoner["id"])
            # Create database row
            leagueoflegends = self.alchemy.get(LeagueOfLegends)(
                region=self.config["Lol"]["region"],
                user=author,
                profile_icon_id=summoner["profileIconId"],
                summoner_name=summoner["name"],
                puuid=summoner["puuid"],
                summoner_level=summoner["summonerLevel"],
                summoner_id=summoner["id"],
                account_id=summoner["accountId"],
                rank_soloq=soloq,
                rank_flexq=flexq,
                rank_twtrq=twtrq,
                rank_tftq=tftq,
                mastery_score=mastery
            )
            log.debug(f"Saving to the DB: {name}")
            data.session.add(leagueoflegends)
            await data.session_commit()
            await data.reply(f"↔️ Account {leagueoflegends} connesso a {author}!")
            await FiorygiTransaction.spawn_fiorygi(data, author, 1,
                                                   "aver connesso il proprio account di League of Legends a Royalnet")
        else:
            # Update and display the League of Legends stats for the current account
            if len(author.leagueoflegends) == 0:
                raise rc.UserError("Nessun account di League of Legends trovato.")
            message = ""
            for account in author.leagueoflegends:
                try:
                    await self._update(account)
                    message += self._display(account)
                except riotwatcher.ApiError as e:
                    message += f"⚠️ [b]{account.summoner_name}[/b]\n" \
                               f"{e}"
                message += "\n"
            await data.session_commit()
            await data.reply(message)



from .leaguetier import LeagueTier
from .leaguerank import LeagueRank


class LeagueLeague:
    def __init__(self,
                 tier: LeagueTier = None,
                 rank: LeagueRank = None,
                 points: int = None,
                 wins: int = None,
                 losses: int = None,
                 inactive: bool = None,
                 hot_streak: bool = None,
                 fresh_blood: bool = None,
                 veteran: bool = None):
        self.tier: LeagueTier = tier  # IRON
        self.rank: LeagueRank = rank  # I
        self.points: int = points  # 40 LP
        self.wins: int = wins
        self.losses: int = losses
        self.inactive: bool = inactive
        self.hot_streak: bool = hot_streak
        self.fresh_blood: bool = fresh_blood
        self.veteran: bool = veteran

    def __str__(self) -> str:
        emojis = ""
        if self.veteran:
            emojis += "🏆"
        if self.hot_streak:
            emojis += "🔥"
        if self.fresh_blood:
            emojis += "⭐️"
        return f"[b]{self.tier} {self.rank}[/b] ({self.points} LP){' ' if emojis else ''}{emojis}"

    def __repr__(self) -> str:
        return f"<{self.__class__.__qualname__} {self}>"

    def __eq__(self, other) -> bool:
        if other is None:
            return False
        if not isinstance(other, LeagueLeague):
            raise TypeError(f"Can't compare {self.__class__.__qualname__} with {other.__class__.__qualname__}")
        equal = True
        if other.veteran:
            equal &= self.veteran == other.veteran
        if other.fresh_blood:
            equal &= self.fresh_blood == other.fresh_blood
        if other.hot_streak:
            equal &= self.hot_streak == other.hot_streak
        if other.inactive:
            equal &= self.inactive == other.inactive
        if other.losses:
            equal &= self.losses == other.losses
        if other.wins:
            equal &= self.wins == other.wins
        if other.points:
            equal &= self.points == other.points
        if other.rank:
            equal &= self.rank == other.rank
        if other.tier:
            equal &= self.tier == other.tier
        return equal

    def __ne__(self, other) -> bool:
        return not self.__eq__(other)

    def __gt__(self, other) -> bool:
        if other is None:
            return True
        if not isinstance(other, LeagueLeague):
            raise TypeError(f"Can't compare {self.__class__.__qualname__} with {other.__class__.__qualname__}")
        if self.empty:
            return False
        elif other.empty:
            return True
        if not (bool(self) and bool(other)):
            raise ValueError("Can't compare partial LeagueLeagues.")
        if self.tier != other.tier:
            # Silver is better than Bronze
            return self.tier > other.tier
        elif self.rank != other.rank:
            # Silver I is better than Silver IV
            return self.rank > other.rank
        elif self.points != other.points:
            # Silver I (100 LP) is better than Silver I (0 LP)
            return self.points > other.points
        elif self.winrate != other.winrate:
            # Silver I (100 LP with 60% winrate) is better than Silver I (100 LP with 40% winrate)
            return self.winrate > other.winrate
        else:
            return False

    def __bool__(self):
        result = True
        result &= self.veteran is not None
        result &= self.fresh_blood is not None
        result &= self.hot_streak is not None
        result &= self.inactive is not None
        result &= self.losses is not None
        result &= self.wins is not None
        result &= self.points is not None
        result &= self.rank is not None
        result &= self.tier is not None
        return result

    @property
    def empty(self):
        result = True
        result &= self.veteran is None
        result &= self.fresh_blood is None
        result &= self.hot_streak is None
        result &= self.inactive is None
        result &= self.losses is None
        result &= self.wins is None
        result &= self.points is None
        result &= self.rank is None
        result &= self.tier is None
        return result

    def __composite_values__(self):
        return self.tier, \
               self.rank, \
               self.points, \
               self.wins, \
               self.losses, \
               self.inactive, \
               self.hot_streak, \
               self.fresh_blood, \
               self.veteran

    @property
    def played(self):
        return self.wins + self.losses

    @property
    def winrate(self):
        return self.wins / self.played

    @classmethod
    def from_dict(cls, d: dict):
        return cls(
            tier=LeagueTier.from_string(d["tier"]),
            rank=LeagueRank.from_string(d["rank"]),
            points=d["leaguePoints"],
            wins=d["wins"],
            losses=d["losses"],
            inactive=d["inactive"],
            hot_streak=d["hotStreak"],
            fresh_blood=d["freshBlood"],
            veteran=d["veteran"],
        )



import enum


class LeagueTier(enum.Enum):
    IRON = 0
    BRONZE = 1
    SILVER = 2
    GOLD = 3
    PLATINUM = 4
    DIAMOND = 5
    MASTER = 6
    GRANDMASTER = 7
    CHALLENGER = 8

    def __str__(self):
        return self.name.capitalize()

    def __repr__(self):
        return f"{self.__class__.__qualname__}.{self.name}"

    def __gt__(self, other):
        return self.value > other.value

    @classmethod
    def from_string(cls, string: str):
        return cls.__members__.get(string)



from typing import *
from .lazyplay import LazyplayCommand


class LazygooglevideoCommand(LazyplayCommand):
    name: str = "lazygooglevideo"

    aliases = ["lgv"]

    description: str = "Cerca un video su Google Video e lo aggiunge (lazy) alla coda della chat vocale."

    syntax = "{ricerca}"

    async def get_urls(self, args):
        return [f"gvsearch:{args.joined()}"]

    # Too bad gvsearch: always finds nothing.



import uuid
import royalnet.utils as ru
from royalnet.backpack.tables import *
from royalnet.constellation.api import *
from ..tables import WikiPage


class ApiWikiEditStar(ApiStar):
    path = "/api/wiki/edit/v1"

    methods = ["POST"]

    summary = "Edit the contents of a wiki page, or create a new one."

    parameters = {
        "id": "The id of the wiki page to edit. Leave empty to create a new one.",
        "title": "The new title of the wiki page.",
        "contents": "The new contents of the wiki page.",
        "format": "The format of the wiki page. The default is markdown.",
        "theme": "The theme of the wiki page. The default is default."
    }

    requires_auth = True

    tags = ["wiki"]

    async def api(self, data: ApiData) -> ru.JSON:
        page_id = data.get("id")
        title = data["title"]
        contents = data["contents"]
        format = data["format"]
        theme = data["theme"]

        WikiPageT = self.alchemy.get(WikiPage)

        user = await data.user()
        if not (user.role == "Admin" or user.role == "Member" or user.role == "Bot"):
            raise ForbiddenError("You do not have sufficient permissions to edit this page.")

        if page_id is None:
            page = WikiPageT(
                page_id=uuid.uuid4(),
                title=title,
                contents=contents,
                format=format,
                theme=theme
            )
            data.session.add(page)
        else:
            page = await ru.asyncify(
                data.session
                    .query(WikiPageT)
                    .filter_by(page_id=uuid.UUID(page_id))
                    .one_or_none
            )
            if page is None:
                raise NotFoundError(f"No page with the id {repr(page_id)} found.")
            page.title = title
            page.contents = contents
            page.format = format
            page.theme = theme

        await data.session_commit()
        return page.json_full()



from typing import *
from royalnet.constellation.api import *
import logging


log = logging.getLogger(__name__)


class ApiDiscordPlayStar(ApiStar):
    path = "/api/discord/play/v1"

    summary = "Add a audio file to the RoyalQueue of a Discord Guild."

    parameters = {
        "url": "The url of the audio file to add.",
        "user": "The name to display in the File Added message.",
        "guild_id": "The id of the guild owning the RoyalQueue to add the audio file to."
    }

    tags = ["discord"]

    async def api(self, data: ApiData) -> dict:
        url = data["url"]
        user = data.get("user")
        guild_id_str = data.get("guild_id")
        if guild_id_str:
            try:
                guild_id: Optional[int] = int(guild_id_str)
            except (ValueError, TypeError):
                raise InvalidParameterError("'guild_id' is not a valid int.")
        else:
            guild_id = None
        log.info(f"Received request to play {url} on guild_id {guild_id} via web")
        response = await self.interface.call_herald_event("discord", "discord_play",
                                                          urls=[url],
                                                          guild_id=guild_id,
                                                          user=user)
        return response



from royalnet.utils import *
from royalnet.backpack.tables import *
from royalnet.constellation.api import *
from ..utils import find_user_api


class ApiBioGetStar(ApiStar):
    path = "/api/bio/get/v1"

    summary = "Get the bio of a Royalnet user."

    parameters = {
        "id": "The user to get the bio of."
    }

    tags = ["bio"]

    async def api(self, data: ApiData) -> dict:
        user = await find_user_api(data["id"], self.alchemy, data.session)
        if user.bio is None:
            raise NotFoundError("User has no bio set.")
        return user.bio.json()



from typing import *
import royalnet
import royalnet.commands as rc
import royalnet.utils as ru
from ..tables import Treasure, FiorygiTransaction


class TreasureCommand(rc.Command):
    name: str = "treasure"

    description: str = "Riscatta un Treasure che hai trovato da qualche parte."

    syntax: str = "{code}"

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        author = await data.get_author(error_if_none=True)
        code = args[0]

        TreasureT = self.alchemy.get(Treasure)

        treasure = await ru.asyncify(data.session.query(TreasureT).get, code)
        if treasure is None:
            raise rc.UserError("Non esiste nessun Treasure con quel codice.")
        if treasure.redeemed_by is not None:
            raise rc.UserError(f"Quel tesoro è già stato riscattato da {treasure.redeemed_by}.")

        treasure.redeemed_by = author
        await data.session_commit()
        await FiorygiTransaction.spawn_fiorygi(data,
                                               author,
                                               treasure.value,
                                               f'aver trovato il tesoro "{treasure.code}"')
        await data.reply("🤑 Tesoro riscattato!")



from typing import *
import random
import royalnet.commands as rc


class SmecdsCommand(rc.Command):
    name: str = "smecds"

    aliases = ["secondomeecolpadellostagista"]

    description: str = "Secondo me, è colpa dello stagista..."

    _DS_LIST = ["della secca", "del seccatore", "del secchiello", "del secchio", "del secchione", "del secondino",
                "del sedano", "del sedativo", "della sedia", "del sedicente", "del sedile", "della sega", "del segale",
                "della segatura", "della seggiola", "del seggiolino", "della seggiovia", "della segheria",
                "del seghetto", "del segnalibro", "del segnaposto", "del segno", "del segretario", "della segreteria",
                "del seguace", "del segugio", "della selce", "della sella", "della selz", "della selva",
                "della selvaggina", "del semaforo", "del seme", "del semifreddo", "del seminario", "della seminarista",
                "della semola", "del semolino", "del semplicione", "della senape", "del senatore", "del seno",
                "del sensore", "della sentenza", "della sentinella", "del sentore", "della seppia", "del sequestratore",
                "della serenata", "del sergente", "del sermone", "della serpe", "del serpente", "della serpentina",
                "della serra", "del serraglio", "del serramanico", "della serranda", "della serratura", "del servitore",
                "della servitù", "del servizievole", "del servo", "del set", "della seta", "della setola", "del sigaro",
                "del sidecar", "del siderurgico", "del sidro", "della siepe", "del sifone", "della sigaretta",
                "del sigillo", "della signora", "della signorina", "del silenziatore", "della silhouette",
                "del silicio",
                "del silicone", "del siluro", "della sinagoga", "della sindacalista", "del sindacato", "del sindaco",
                "della sindrome", "della sinfonia", "del sipario", "del sire", "della sirena", "della siringa",
                "del sismografo", "del sobborgo", "del sobillatore", "del sobrio", "del soccorritore", "del socio",
                "del sociologo", "della soda", "del sofà", "della soffitta", "del software", "dello sogghignare",
                "del soggiorno", "della sogliola", "del sognatore", "della soia", "del solaio", "del solco",
                "del soldato", "del soldo", "del sole", "della soletta", "della solista", "del solitario",
                "del sollazzare", "del sollazzo", "del sollecito", "del solleone", "del solletico", "del sollevare",
                "del sollievo", "del solstizio", "del solubile", "del solvente", "della soluzione", "del somaro",
                "del sombrero", "del sommergibile", "del sommo", "della sommossa", "del sommozzatore", "del sonar",
                "della sonda", "del sondaggio", "del sondare", "del sonnacchioso", "del sonnambulo", "del sonnellino",
                "del sonnifero", "del sonno", "della sonnolenza", "del sontuoso", "del soppalco", "del soprabito",
                "del sopracciglio", "del sopraffare", "del sopraffino", "del sopraluogo", "del sopramobile",
                "del soprannome", "del soprano", "del soprappensiero", "del soprassalto", "del soprassedere",
                "del sopravvento", "del sopravvivere", "del soqquadro", "del sorbetto", "del sordido", "della sordina",
                "del sordo", "della sorella", "della sorgente", "del sornione", "del sorpasso", "della sorpresa",
                "del sorreggere", "del sorridere", "della sorsata", "del sorteggio", "del sortilegio",
                "del sorvegliante", "del sorvolare", "del sosia", "del sospettoso", "del sospirare", "della sosta",
                "della sostanza", "del sostegno", "del sostenitore", "del sostituto", "del sottaceto", "della sottana",
                "del sotterfugio", "del sotterraneo", "del sottile", "del sottilizzare", "del sottintendere",
                "del sottobanco", "del sottobosco", "del sottomarino", "del sottopassaggio", "del sottoposto",
                "del sottoscala", "della sottoscrizione", "del sottostare", "del sottosuolo", "del sottotetto",
                "del sottotitolo", "del sottovalutare", "del sottovaso", "della sottoveste", "del sottovuoto",
                "del sottufficiale", "della soubrette", "del souvenir", "del soverchiare", "del sovrano",
                "del sovrapprezzo", "della sovvenzione", "del sovversivo", "del sozzo", "dello suadente", "del sub",
                "del subalterno", "del subbuglio", "del subdolo", "del sublime", "del suburbano", "del successore",
                "del succo", "della succube", "del succulento", "della succursale", "del sudario", "della sudditanza",
                "del suddito", "del sudicio", "del suffisso", "del suffragio", "del suffumigio", "del suggeritore",
                "del sughero", "del sugo", "del suino", "della suite", "del sulfureo", "del sultano", "di Steffo",
                "di Spaggia", "di Sabrina", "del sas", "del ses", "del sis", "del sos", "del sus", "della supremazia",
                "del Santissimo", "della scatola", "del supercalifragilistichespiralidoso", "del sale", "del salame",
                "di (Town of) Salem", "di Stronghold", "di SOMA", "dei Saints", "di S.T.A.L.K.E.R.", "di Sanctum",
                "dei Sims", "di Sid", "delle Skullgirls", "di Sonic", "di Spiral (Knights)", "di Spore", "di Starbound",
                "di SimCity", "di Sensei", "di Ssssssssssssss... Boom! E' esploso il dizionario", "della scala",
                "di Sakura", "di Suzie", "di Shinji", "del senpai", "del support", "di Superman", "di Sekiro",
                "dello Slime God", "del salassato", "della salsa", "di Senjougahara", "di Sugar", "della Stampa",
                "della Stampante"]

    async def run(self, args: rc.CommandArgs, data: rc.CommandData) -> None:
        ds = random.sample(self._DS_LIST, 1)[0]
        await data.reply(f"🤔 Secondo me, è colpa {ds}.")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['royalpack',
 'royalpack.commands',
 'royalpack.events',
 'royalpack.stars',
 'royalpack.tables',
 'royalpack.types',
 'royalpack.utils']

package_data = \
{'': ['*'], 'royalpack': ['pycharm/*']}

install_requires = \
['riotwatcher>=2.7.1,<3.0.0',
 'royalnet[telegram,discord,alchemy_easy,bard,constellation,sentry,herald,coloredlogs]>=5.8.7,<5.9.0',
 'royalspells>=3.2,<4.0',
 'steam']

setup_kwargs = {
    'name': 'royalpack',
    'version': '5.8.9',
    'description': 'A Royalnet command pack for the Royal Games community',
    'long_description': '# `royalpack`\n\n## Required configuration options\n\n```toml\nImgur.token = \nTelegram.main_group_id = \nDiscord.main_channel_id = \nDota.updater = \nPeertube.instance_url = \nPeertube.feed_update_timeout = \nFunkwhale.instance_url = \nCv.displayed_role_id = \nLol.token = \nLol.region = \nLol.updater = \nPlay.max_song_duration = \nSteam.web_api_key = \nBrawlhalla.api_key = \nBrawlhalla.updater = \nMatchmaking.mm_chat_id = \n```',
    'author': 'Stefano Pigozzi',
    'author_email': 'ste.pigozzi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Steffo99/royalpack',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
